#!/usr/bin/env python 
"""
Copyright (C) 2009 Nick Drobchenko, nick@cnc-club.ru
based on gcode.py (C) 2007 hugomatic... 
based on addnodes.py (C) 2005,2007 Aaron Spike, aaron@ekips.org
based on dots.py (C) 2005 Aaron Spike, aaron@ekips.org

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""



import inkex, simplestyle, simplepath
import os
import math
import bezmisc
import re
import copy
biarc_style = {
		'biarc0':	simplestyle.formatStyle({ 'stroke': '#88f', 'fill': 'none', 'stroke-width':1 }),
		'biarc1':	simplestyle.formatStyle({ 'stroke': '#8f8', 'fill': 'none', 'stroke-width':1 }),
		'line':		simplestyle.formatStyle({ 'stroke': '#f88', 'fill': 'none', 'stroke-width':1 }),
		'area':		simplestyle.formatStyle({ 'stroke': '#777', 'fill': 'none', 'stroke-width':1 }),
	}

straight_tolerance = 0.000001
straight_distance_tolerance = 0.0001
parse_codes_maxdeph = 4
#scalex = scalex*scale 
#scaley = scaley*scale 



defaults = {
		'header': '(Generated by gcode_tools from inkscape.) \nM3 \n',
		'footer': 'M5 \nG00 X0.0000 Y0.0000 \nM2 \n(end)\n'
}


if os.path.isfile("/home/nick/output.txt") :os.remove("/home/nick/output.txt")
def print_(s=''):
	f = open("/home/nick/output.txt","a")
	f.write(str(s))
	f.write("\n")
	f.close()





class P:
	def __init__(self, x, y=None):
		if not y==None:
			self.x, self.y = float(x), float(y)
		else:
			self.x, self.y = float(x[0]), float(x[1])

	def __add__(self, other):
		return P(self.x + other.x, self.y + other.y)

	def __sub__(self, other):
		return P(self.x - other.x, self.y - other.y)

	def __neg__(self):
		return P(-self.x, -self.y)


	def __mul__(self, other):
		if isinstance(other, P):
			return self.x * other.x + self.y * other.y
		return P(self.x * other, self.y * other)
	__rmul__ = __mul__

	def __div__(self, other):
		return P(self.x / other, self.y / other)

	def mag(self):
		return math.hypot(self.x, self.y)

	def unit(self):
		h = self.mag()
		if h: return self / h
		else: return P(0,0)
	
	def dot(self, other):
		return self.x * other.x + self.y * other.y

	def rot(self, theta):
		c = math.cos(theta)
		s = math.sin(theta)

		return P(self.x * c - self.y * s,  self.x * s + self.y * c)

	def angle(self):
		return math.atan2(self.y, self.x)

	def __repr__(self): return '%f,%f' % (self.x, self.y)
	def pr(self): return "%.2f,%.2f" % (self.x, self.y)
	def to_list(self): return [self.x, self.y]	





def tpoint((x1,y1), (x2,y2), t = 0.5):
    return [x1+t*(x2-x1),y1+t*(y2-y1)]

def cspbezsplit( bez, t=0.5):
	m1=tpoint(bez[0],bez[1],t)
	m2=tpoint(bez[1],bez[2],t)
	m3=tpoint(bez[2],bez[3],t)
	m4=tpoint(m1,m2,t)
	m5=tpoint(m2,m3,t)
	m=tpoint(m4,m5,t)
	return  [bez[0],m1,m4,m], [m,m5,m3,bez[3]] 

def cspbezsplitatlength(bez, l = 0.5, tolerance = 0.001):
    t = bezmisc.beziertatlength(bez, l, tolerance)
    return cspbezsplit(bez, t)	

class Gcode(inkex.Effect):

	def __init__(self):
		inkex.Effect.__init__(self)
		self.OptionParser.add_option("-d", "--directory",		action="store", type="string", 		dest="directory", default="/home/",				help="Directory for gcode file")
		self.OptionParser.add_option("-f", "--filename",		action="store", type="string", 		dest="file", default="-1.0",					help="File name")			
		self.OptionParser.add_option("-u", "--Xscale",			action="store", type="float", 		dest="Xscale", default="10.0",					help="Scale factor X")	
		self.OptionParser.add_option("-v", "--Yscale",			action="store", type="float", 		dest="Yscale", default="-10.0",					help="Scale factor Y")	
		self.OptionParser.add_option("-x", "--Xoffset",			action="store", type="float", 		dest="Xoffset", default="0.0",					help="Offset along X")	
		self.OptionParser.add_option("-y", "--Yoffset",			action="store", type="float", 		dest="Yoffset", default="11.0",					help="Offset along Y")
		self.OptionParser.add_option("-s", "--Zsafe",			action="store", type="float", 		dest="Zsafe", default="0.5",					help="Z above all obstacles")
		self.OptionParser.add_option("-z", "--Zsurface",		action="store", type="float", 		dest="Zsurface", default="0.0",					help="Z of the surface")
		self.OptionParser.add_option("-c", "--Zdeph",			action="store", type="float", 		dest="Zdeph", default="-0.125",					help="Z depth of cut")
		self.OptionParser.add_option("",   "--Zstep",			action="store", type="float", 		dest="Zstep", default="-0.125",					help="Z step of cutting")		
		self.OptionParser.add_option("-p", "--feed",			action="store", type="float", 		dest="feed", default="4.0",						help="Feed rate in unit/min")
		self.OptionParser.add_option("",   "--cut-area",		action="store", type="string", 		dest="cut_area", default="false",				help="Cut internal area of the curve")				
		self.OptionParser.add_option("",   "--tool-diameter",	action="store", type="float", 		dest="tool_diameter", default="3",				help="Tool diameter used for area cutting")		
		self.OptionParser.add_option("",   "--max-area-curves",	action="store", type="int", 		dest="max_area_curves", default="100",			help="Maximum area curves for each area")
		self.OptionParser.add_option("",   "--unit",			action="store", type="string", 		dest="unit", default="G21 (All units in mm)",	help="Units")




	def biarc(self, bez):
		P0, P4 = P(bez[0]), P(bez[3])
		TS, TE, v = (P(bez[1])-P0), -(P(bez[2])-P4), P0 - P4
		tsa, tea, va = TS.angle(), TE.angle(), v.angle()
		if TE.mag()<straight_distance_tolerance and TS.mag()<straight_distance_tolerance:	
			# Both tangents are zerro - line straight
			return ['Add L']
		if TE.mag() == 0:
			TE = -(TS+v).unit()
			r = min(.5, TS.mag()/v.mag())
		elif TS.mag() == 0:
			TS = -(TE+v).unit()
			r = max(.5, TE.mag()/v.mag())
		else:	
			r=(P(bez[1])-P0).mag()/(P(bez[2])-P4).mag()
		TS, TE = TS.unit(), TE.unit()
		
		if  1-abs(TS*TE/(TS.mag()*TE.mag()))<straight_tolerance and v.mag()<straight_distance_tolerance:
			# Both tangents are parallel and start and end are the same - line straight
			return ['Add L']
		if 	1-abs(TS*TE/(TS.mag()*TE.mag()))<straight_tolerance and  1-abs(TS*v/(TS.mag()*v.mag()))<straight_tolerance :
			# Both tangents and v are parallel - line straight
			return ['Add L']  				
		
		c,b,a = v*v, 2*v*(r*TS+TE), 2*r*(TS*TE-1)

		if v.mag()==0 and abs(TE.angle())!=abs(TS.angle()):
			return ['Need split']
		if 		a == 0 and b!=0:	beta = -c/b
		elif 	c == 0 and a!=0:	beta = -b/a 
		elif	a!=0 and b!=0:	 
			discr = b*b-4*a*c
			if discr < 0:	raise ValueError, (a,b,c,discr)
			disq = discr**.5
			beta1 = (-b - disq) / 2 / a
			beta2 = (-b + disq) / 2 / a
			if beta1*beta2 > 0 :	raise ValueError, (a,b,c,disq,beta1,beta2)
			beta = max(beta1, beta2)
		elif	a==0 and b==0:	raise ValueError, ("a==0 and b==0")
		alpha = beta * r
		ab = alpha + beta 
		P1 = P0 + alpha * TS
		P3 = P4 - beta * TE
		P2 = (beta / ab)  * P1 + (alpha / ab) * P3
		return ['Biarc', [ [P0,P1,P2,P3,P4], alpha, beta ] ]



	
	def parse_code(self, cmd, params, cpos, deph=1):
		if cmd == 'M':
			self.current_curve_moved_from = params[-2:]
			return params[-2:], [[cpos, 'move', 0, 0]]
		if cmd == 'C':
			bez = ( cpos, params[:2], params[2:4], params[-2:] )
			b = self.biarc(bez)
			if b[0] == "Add L":
				cmd = 'L'
			elif b[0] == 'Need split':
				if deph>parse_codes_maxdeph:
					cmd = 'L'
				else:	
					p=[0]*6
					bez1, bez2 = cspbezsplitatlength(bez)
					(p[0], p[1]), (p[2], p[3]), (p[4], p[5]) = bez1[1:]
					cpos, curve = self.parse_code('C',p,cpos, deph+1)
					(p[0], p[1]), (p[2], p[3]), (p[4], p[5]) = bez2[1:]
					cpos, curve1 = self.parse_code('C',p,cpos, deph+1)
					curve.extend(curve1)
					return cpos, curve
			elif b[0] == 'Biarc':
				def calculate_arc(P0,P1,P2):
					D = (P0+P2)/2
					if (D-P1).mag()==0: return [ [P0.x,P0.y], 'line' ]
					R = D - ( (D-P0).mag()**2/(D-P1).mag() )*(P1-D).unit()
					p0a, p1a, p2a = (P0-R).angle()%(2*math.pi), (P1-R).angle()%(2*math.pi), (P2-R).angle()%(2*math.pi)
					alpha =  (p2a - p0a) % (2*math.pi)					
					if (p0a<p2a and  (p1a<p0a or p2a<p1a))	or	(p2a<p1a<p0a) : 
						alpha = -2*math.pi+alpha
					return  [[P0.x,P0.y], 'arc', [R.x,R.y], alpha ] 
					
				return [b[1][0][4].x,b[1][0][4].y], [calculate_arc(b[1][0][0], b[1][0][1], b[1][0][2]), calculate_arc(b[1][0][2], b[1][0][3], b[1][0][4])]
				
		if cmd == 'Z': 
			return params[-2:], [[cpos, 'line',0 , 0], [self.current_curve_moved_from, 'end']]
			
		if cmd == 'L':
			return params[-2:], [[cpos, 'line',0 , 0]]
		


	def draw_curve(self, curve, group):
###		try:
###			t = node.get('transform')
###			self.group.set('transform', t)
###		except:
###			pass
		s, arcn = '', 0
		for si in curve:
			if s!='':
				if s[1] == 'line':
					inkex.etree.SubElement(	group, inkex.addNS('path','svg'), 
							{
								'style':biarc_style['line'],
								'd':'M %s,%s L %s,%s' % (s[0][0], s[0][1], si[0][0], si[0][1]),
							}
						)
				elif s[1] == 'arc':
					arcn += 1
					sp = s[0]
					c = s[2]
					a = s[3]
					r = math.sqrt( (sp[0]-c[0])**2 + (sp[1]-c[1])**2 )
					a_st = ( math.atan2(sp[0]-c[0],- (sp[1]-c[1])) - math.pi/2 ) % (math.pi*2)
					if a>0:
						a_end = a_st+a
					else: 
						a_end = a_st*1
						a_st = a_st+a	
					inkex.etree.SubElement(	group, inkex.addNS('path','svg'), 
						 {
							'style': biarc_style['biarc%s' % (arcn%2)],
							 inkex.addNS('cx','sodipodi'):		str(c[0]),
							 inkex.addNS('cy','sodipodi'):		str(c[1]),
							 inkex.addNS('rx','sodipodi'):		str(r),
							 inkex.addNS('ry','sodipodi'):		str(r),
							 inkex.addNS('start','sodipodi'):	str(a_st),
							 inkex.addNS('end','sodipodi'):		str(a_end),
							 inkex.addNS('open','sodipodi'):	'true',
							 inkex.addNS('type','sodipodi'):	'arc',
						})
			s = si
					
		
		
	def offset_curve(self,curve,d):
		p1 = []
		def offset(s, si, d):
			s = copy.copy(s)
			if s[1] == 'line':
				sp = P(s[0])
				n = P(s[0][1] - si[0][1], si[0][0] - s[0][0]).unit()*d
				s[0] = [s[0][0] + n.x, s[0][1]+n.y]
				s[2] = [si[0][0] + n.x, si[0][1]+n.y]
			if s[1] == 'arc':
				n = P(s[2][0] - s[0][0], s[2][1] - s[0][1]).unit()*d
				if s[3] < 0 :
					n = -n  
				s[0] = [s[0][0] + n.x, s[0][1]+n.y]
			return s
			
			
		def find_intersections(s,si):
			if s[1]==si[1]:
				if s[1] == 'line':				
					sx, sy = s[2][0]-s[0][0], s[2][1]-s[0][1]
					six, siy = si[2][0]-si[0][0], si[2][1]-si[0][1]
					if (sx==0 and sy==0) or (six==0 and siy==0) : return []
					if sx==six==0 :
						if sy != siy : return []
						else : return ['Overlap'] # Overlap
					if (six*sy==sx*siy): 
						if ( sy*(si[0][0] - s[0][0]) == sx*(si[0][1]-s[0][1]) ) : return ['Overlap']  # Overlap
						else : return []
					if sx == 0 :
						x = s[0][0] 
						if siy == 0 : y = si[0][1]
						else : y = x * siy/six - si[0][0]*siy/six + si[0][1]
					elif six == 0:
						x = si[0][0] 
						if sy == 0 : y = s[0][1]
						else : y = x * sy/sx - s[0][0]*sy/sx + s[0][1]
					else:	
						a, ai = sy/sx, siy/six
						b, bi = - s[0][0]*sy/sx + s[0][1], - si[0][0]*siy/six + si[0][1]
						if bi-b != 0:						
							x = (bi-b)/(a-ai)
						else: x = 0	
						y = a*x+b
					return [[x,y]]
				elif s[1]== 'arc' :
					s = copy.copy(s)
					s[1] = 'line_abc'
					r2, ri2 = (s[0][0]-s[2][0])**2 + (s[0][1]-s[2][1])**2, (si[0][0]-si[2][0])**2 + (si[0][1]-si[2][1])**2
					x1,y1 = si[2][0], si[2][1]
					x2,y2 = s[2][0] - x1, s[2][1] - y1
					A = -2*(x2)
					B = -2*(y2)
					C = (x2)**2+(y2)**2 - r2 + ri2 - A*x1-B*y1
					s[0] = [A,B,C]
				
			if si[1]!=s[1] and si[1] in ['line','line_abc','arc'] and s[1] in ['line','line_abc','arc'] :
				
				if si[1] == 'arc' : a, l = si, s
				else :	a, l = s, si
				if l[1]=='line_abc': 
					A,B,C = l[0]
					C += A*a[2][0] + B*a[2][1]
				else :
					A, B = l[2][1] - l[0][1], - ( l[2][0] - l[0][0])
					C =  - l[0][0]*A - l[0][1]*B  + A*a[2][0] + B*a[2][1]
				r2 = (a[0][0]-a[2][0])**2 + (a[0][1]-a[2][1])**2
				x0, y0 = -A*C/(A**2+B**2), -B*C/(A**2+B**2)
				if 	abs(A*x0+B*y0+C)>abs(-A*x0-B*y0+C) : x0, y0 = -x0,-y0
				if abs (C**2 - r2*(A**2+B**2)) < straight_distance_tolerance : 
					return [[x0+a[2][0], y0+a[2][1]]]
				elif C**2 > r2*(A**2+B**2) : 
					return []
				else :

					m = math.sqrt( (r2 - C**2/(A**2+B**2)) / (A**2+B**2) )
					return [[a[2][0]+x0-B*m, a[2][1]+y0+A*m], [a[2][0]+x0+B*m, a[2][1]+y0-A*m]]
					
			else :	
				return []

		def between(c,x,y):
			return x<=c<=y or y<=c<=x
		
		for i in range(len(curve)-1):
			if i==0 :
				p1.extend([offset(curve[i],curve[i+1],d)])
			else :					
				s = p1[-1]
				si = offset(curve[i],curve[i+1],d)
				
				int_ = find_intersections(s,si)
				if len(int_)>1:	
					for k in range(len(int_)):
						int_[k] = [ (curve[i][0][0]-int_[k][0])**2 + (curve[i][0][1]-int_[k][1])**2, int_[k]]
					int_.sort()
					int_ = int_[0][1]
				elif int_ != [] : int_ = int_[0]
				
				if s[1]==si[1]=='line':
					x1,y1,x2,y2 = s[0][0], s[0][1], s[2][0], s[2][1]
					xi1,yi1,xi2,yi2 = si[0][0], si[0][1], si[2][0], si[2][1]
					
					if int_ != []:
						if int_ == ['Overlap']:
							p1.extend([ [s[2],'line',si[2]] ])
						else:
							ip={}
							if between(int_[0],x1,x2) and between(int_[1],y1,y2): ip['s']='t'
							elif between(x2,x1,int_[0]): ip['s']='fp'
							else: ip['s']='fn'
							if between(int_[0],xi1,xi2) and between(int_[1],yi1,yi2): ip['si']='t'
							elif between(xi2,xi1,int_[0]): ip['si']='fp'
							else : ip['si']='fn'
								
							if ( ip['s']=='t' and ip['si']=='t'  ):
								p1.extend([ [int_,'line',si[2]] ])
							elif ( ip['s']!='t' and ip['si']!='t' ):
								if ip['s']=='fp' : 
									p1.extend([ [int_,'line',si[2]] ])
								else: p1.extend([ [s[2],'line',int_],[int_,'line',si[2]] ])
							elif ( ip['s']=='t' or ip['si']=='t' ):
								p1.extend([ [s[2],'line',int_],[int_,'line',si[2]] ])
					if i == len(curve)-2:
						p1.extend([ [si[2],'end'] ])

				elif s[1]=='line' and si[1]=='arc':
					if int_ != []:
					
						x1,y1,x2,y2 = s[0][0], s[0][1], s[2][0], s[2][1]
						ip={}
						if between(int_[0],x1,x2) and between(int_[1],y1,y2): ip['s']='t'
						elif between(x2,x1,int_[0]): ip['s']='fp'
						elif between(x2,x1,int_[0]): ip['s']='fn'	
							
						P0, R, Pi = P(si[0]), P(si[2]), P(int_)
						alpha =  ((Pi-R).angle() - (P0-R).angle()) 

						if between(alpha,0,si[3]) : ip['si'] = 't'
						else : ip['si'] = 'f'
							
						if (ip['s']=='t' and ip['si']=='t'):
							p1.extend([ [int_,'arc', si[2], si[3]-alpha] ])
						elif (ip['s']=='fp' and ip['si']!='t'):
							c = P(curve[i][0])
							ps, pe = P(s[2])-c, P(si[0])-c	
							
							ang = pe.angle()-ps.angle()
							if ((ps-c).rot(ang)+c-pe).mag() < ((ps-c).rot(-ang)+c-pe).mag():
								ang=ang-2*math.pi
							print_([s[2], 'arc', [c.x,c.y], ang, alpha])	
							p1.extend([   [s[2], 'arc', [c.x,c.y], ang] , 
							
										[si[0],'arc',si[2], si[3]]	])
						elif (ip['s']!='t' and ip['si']=='t') or (ip['s']=='t' and ip['si']!='t'):
							p1.extend([  [s[2],'line',si[0]], si  ])
					else:
						c = curve[i+1][0]
						p1.extend([   [s[2], 'arc', c, ( ((P(s[2])-P(c)).angle()-(P(si[0])-P(c)).angle()) )%(2*math.pi)-math.pi],
									[si[0],'arc',si[2], si[3]]	])
					if i == len(curve)-2:
						p1.extend([ [si[2],'end'] ])			
				
				else:
					p1.extend([ si ])
		
				
		
#		p1.extend([curve[-1]])
		for i in range(len(p1)-1):
			inter = find_intersections(p1[i],p1[i+1])
#			for c in inter:
#				inkex.etree.SubElement(	self.biarcGroup, inkex.addNS('path','svg'), 
#					 {
#						'style': biarc_style['biarc1'],
#						 inkex.addNS('cx','sodipodi'):		str(c[0]),
#						 inkex.addNS('cy','sodipodi'):		str(c[1]),
#						 inkex.addNS('rx','sodipodi'):		str(1),
#						 inkex.addNS('ry','sodipodi'):		str(1),
#						 inkex.addNS('type','sodipodi'):	'arc',
#					})
			
		return p1

		
		
	###
	###		Curve structure	
	###		[	[s - start point, type - line|arc|end|move, c - center or next point(optional), a - alpha [-2pi,2pi] illustrates direction] ...	]
	###
	def parse_curve(self, d):
		p = simplepath.parsePath(d)
		cpos = [0,0]
		curve = []
		curves = []
		for cmd,params in p:
			cpos, c = self.parse_code(cmd,params,cpos, deph=1)
			curve.extend(c)
			if curve[-1:][0] == 'end':
				curves.extend(curve)
				curve = []
		if curve != []:
			curve.extend([ [cpos, 'end'] ])
			curves.extend([curve])
		return curves
		


	def generate_gcode(self, curve):
		pass
	###		Move	
	#	self.gcode += 'G1 Z%f F%f\n' % (self.options.Zsafe, self.options.feed) +  'G0 X%f Y%f \n' % (code[1][-2], code[1][-1]) + 'G1 Z%f \n' % (Zpos)
	###		Line	
	# 	["G-code", "G1 X%f Y%f" % (params[-2],params[-1])], params[-2:]
		
					
	def effect(self):
		if (os.path.isdir(self.options.directory)):
			if (os.path.isfile(self.options.directory+'/header')):
				f = open(self.options.directory+'/header', 'r')
				self.header = f.read()
				f.close()
			else:
				self.header = defaults['header']
			if (os.path.isfile(self.options.directory+'/footer')):
				f = open(self.options.directory+'/footer','r')
				self.footer = f.read()
				f.close()
			else:
				self.footer = defaults['footer']
		else: return
		self.header += self.options.unit
		self.final_gcode = self.header
		

		for id, node in self.selected.iteritems():
			self.biarcGroup = inkex.etree.SubElement( node.getparent(), inkex.addNS('g','svg') )
			break

		
		for id, node in self.selected.iteritems():
			if node.tag == inkex.addNS('path','svg'):
				self.gcode = '(Found curve %s)' % node.get('id').replace('(','').replace(')','').replace('\n','')
				original_d = node.get('d')
	###
	###		Calculating new curve for the path.
	###		
				if self.options.Zstep == 0 : Zstep = 1
				for step in range( 0,  int(math.ceil( abs( (self.options.Zdeph-self.options.Zsurface)/self.options.Zstep )) ) ):
	###
	###		This is unoptimal for loop invokes all calculations for each step. 
	###		It's done so to simplify futher improve for variable Zposition.
	###				
					Zpos = max(		self.options.Zdeph,		 self.options.Zsurface - abs(self.options.Zstep*(step+1))	)
					self.gcode += 'G0 Z%f \n'
	###
	###		Compute area cutting if needed
	###
#					if self.options.cut_area=='true' :
#						if self.options.tool_diameter<=0 : 
#							inkex.errormsg(_("Tool diameter must be > 0!"))				
#							return
#						i = 0
#						area_curves = ''
#						while i==0 or (i<self.options.max_area_curves and loop):
#							i += 1
#							arc_attr = {
#								inkex.addNS('type','sodipodi'):		'inkscape:offset',
#								inkex.addNS('radius','inkscape'):	str(-self.options.tool_diameter*i ),
#								inkex.addNS('original','inkscape'):	original_d,
#										}
#
#										
#							area_curve = inkex.etree.SubElement(
#								self.biarcGroup,
#								inkex.addNS('path','svg'), arc_attr )
#
#							loop = s == None
#							if s!=None:
#								area_curves += str(s)
							
					curves = self.parse_curve(original_d)
					for curve in curves:
						print_ (curve)
						print_()
						self.draw_curve(curve, self.biarcGroup)
						p1 = self.offset_curve(curve,3)
						self.draw_curve(p1, self.biarcGroup)
				
		self.final_gcode += self.footer
				
		f = open(self.options.directory+'/'+self.options.file, "w")	
		f.write(self.final_gcode)
		f.close()							
									
	
e = Gcode()
e.affect()
